#!/usr/bin/python3

"""
Deezspot models package.
"""

# Import subpackages to ensure they're included
from deezspot.models import callback
from deezspot.models import download
from deezspot.models import librespot
